/**
 * Akeeba Remote Control
 * A utility to remotely control Akeeba Backup and Admin Tools
 * 
 * @version $id
 * @copyright 2008-2010 Nicholas K. Dionysopoulos / AkeebaBackup.com
 * @license GNU/GPL v3 or later
 */

/*
 * Global variables
 */

/** @var Array Contains the list of selected sites' UUIDs */
var akeeba_remote_selected_sites = [];
/** @var Object The application properties object */
var akeeba_remote_properties = null;
/** @var Boolean When this flag is true, Akeeba Remote acts as a portable application */
var akeeba_remote_portable = false;

/**
 * Initialize the application
 */
$(document).ready(function(){
	// Who am I?
	var tiApp = Titanium.API.application;
	$('#idxAppName').text(tiApp.getName());
	$('#idxAppVersion').text(tiApp.getVersion());
	
	// Attach window resize code
	onWindowResize();
	$(window).resize(onWindowResize);
	
	// Turn site list into a selectable list
	makeSiteListSelectable();
	
	// Buttons in Multiple Selection pane
	$('#idxStartBackupAll').click(startAllBackup);
	$('#idxCancelBackupAll').click(cancelAllBackup);
	// Buttons in Loading pane
	$('#idxWaitingEditSite').click( function(){ siteEditorShow(null); } );
	$('#idxWaitingDeleteSite').click(deleteCurrentSite);
	// Buttons in Single Selection pane
	$('#idxBackupNow').click(startCurrentBackup);
	$('#idxCancelBackup').click(cancelCurrentBackup);
	$('#idxCancelDownload').click(cancelCurrentBackup);
	$('#idxAddSite').click( function(){ siteEditorShow(null); } );
	$('#idxEditSite').click(editCurrentSite);
	$('#idxEditClone').click( function(){ cloneSite(null); } );
	$('#idxDeleteSite').click(deleteCurrentSite);
	$('#idxResetSite').click(resetCurrentSiteErrors);
	$('#idxViewLog').click(viewCurrentSiteLog);
	$('#idxViewCurrentLog').click(viewCurrentSiteLog);
	$('#idxDeleteSiteAfterError').click(deleteCurrentSite);
	$('#idxEditSiteAfterError').click(editCurrentSite);
	
	$('#downloadProgress').progressbar({ value: 10 });
	
	// Load app properties (site definitons included!) from settings storage
	loadAppProperties();	
});

/**
 * Loads the application properties from a file
 * @return
 */
function loadAppProperties()
{
	// Init
	akeeba_remote_properties = null;
	
	// Do we have a local properties file in the same directory as the application?
	var rootPath = Titanium.API.application.getPath();
	var file = Titanium.Filesystem.getFile( rootPath, 'application.properties' );
	if( file.exists() ) {
		akeeba_remote_portable = true;
	} else {
		file = Titanium.Filesystem.getFile( Titanium.API.application.dataPath, 'application.properties' );
	}
	
	// Try opening the properties file
	if( file.exists() ) {
		akeeba_remote_properties = Titanium.App.loadProperties(file.nativePath());
	}
	
	// If no properties were loaded, create the default ones
	if(!akeeba_remote_properties) {
		akeeba_remote_properties = Titanium.App.createProperties({
			'sitedefs': "{}",
			'language': "en-GB"
		});
	}
	
	// Unserialize the site information
	var sitedefs = akeeba_remote_properties.getString('sitedefs');
	var loadedSiteDefs = JSON.parse(sitedefs);
	if(!empty(loadedSiteDefs)) {
		$.each(loadedSiteDefs,function(uuid, data){
			siteManager.addSite(uuid,data);
		});
	}
	
	if(empty(akeeba_remote_properties.getString('language'))) akeeba_remote_properties.setString('language','en-GB');
	$('#idxSelectLanguage').val(akeeba_remote_properties.getString('language'));
	Trans.apply($('#idxSelectLanguage').val());
}

/**
 * Saves the application properties to a file
 * @return
 */
function saveAppProperties()
{
	// Property builder
	// -- Serialize the site information
	var sitedefs = siteManager.getSerialized();
	// OK, this is actually absurd... I have to escape backslashes with... backslashes. *sigh*
	sitedefs = str_replace('\\','\\\\',sitedefs);
	akeeba_remote_properties.setString('sitedefs', sitedefs);
	// Get the preferred language
	var language = $('#idxSelectLanguage').val();
	akeeba_remote_properties.setString('language',language);
	
	// Save the properties to the file
	var rootPath = Titanium.API.application.getPath();
	var file = '';
	if( akeeba_remote_portable ) {
		file = Titanium.Filesystem.getFile( rootPath, 'application.properties' );
	} else {
		file = Titanium.Filesystem.getFile( Titanium.API.application.dataPath, 'application.properties' );
	}
	akeeba_remote_properties.saveTo(file.nativePath());
}

/**
 * Funky event listener to always save settings on exit
 */
Titanium.API.addEventListener(Titanium.EXIT, function(event){
	saveAppProperties();
});

/**
 * Window resize event handler
 */
function onWindowResize()
{
	$('#idxMiddle').height( $(window).height() - 78 );
}

/**
 * Applies the search box styling (tentative)
 * TODO Make it functional, too
 * @return
 */
function siteSearchBoxStyling()
{
	//$('#idxSearch').button();
	//$('#idxSearchClear').button();
}

/**
 * Make the site unordered list into a selectable list
 * @return
 */
function makeSiteListSelectable()
{
	$('#siteList').sortable({ handle: 'span.handle' });
	$('#siteList').selectable();
	$('#siteList').bind('selectablestop', onSiteSelectionStop);
}

/**
 * Event callback for site selection change
 * @param event
 * @param ui
 * @return
 */
function onSiteSelectionStop(event,ui)
{
	// Reset the selected sites array
	akeeba_remote_selected_sites = [];
	
	// Push selected UUIDs to the array
	$('#siteList li.ui-selected').each(function(idx, el){
		var siteID = $(el).attr('id');
		var uuid = substr(siteID, 5);
		
		array_push(akeeba_remote_selected_sites, uuid);
	});
	
	// Is it only one selected or many of them?
	if( count(akeeba_remote_selected_sites) > 1 )
	{
		// Is the multisite pane visible already?
		$('#selectedMultiple').css('display','block');
		$('#selectedSingle').css('display','none');
	} else {
		// Is the single site pane visible already?
		$('#selectedMultiple').css('display','none');
		$('#selectedSingle').css('display','block');
		
		// Render the single site pane
		renderSingleSitePane();
	}
}

/**
 * Renders or updates the single site pane
 * @return
 */
function renderSingleSitePane()
{
	var uuid = akeeba_remote_selected_sites[0];
	var siteSelector = '#site-' + uuid;
	
	// Set the name on the pane
	$('#selectedSingle h2').text( $(siteSelector+' span.siteName:first').text() );
	
	// Hide everything
	$('div.idxPleaseWait').css('display','none');
	$('#idxSiteInformation').css('display','none');
	$('#backupStatus').css('display','none');
	$('#downloadInformation').css('display','none');
	$('#errorPane').css('display','none');
	
	// Load the Akeeba API Object of this item
	var akeebaAPI = $(siteSelector).data('akeeba_api_object');
	
	// What is our status?
	var status = siteManager.getSiteStatus(uuid);
	
	switch(status)
	{
		case 'idle':
			// Show the spinner
			$('div.idxPleaseWait').css('display','block');
			akeebaAPI.getVersion(true);
			break;

		case 'queue':
		case 'backup':
			// Render a site backing up
			$('#backupStatus').css('display','block');
			renderSiteBackupPane();
			break;

		case 'download':
			// Render a site downloading
			$('#downloadInformation').css('display','block');
			renderSiteDownloadPane();
			break;

		case 'dlerror':
		case 'error':
		case 'genericerror':
			// Render error messages
			$('#errorPaneContent').html(akeebaAPI.errorMessage);
			$('#errorPane').show();
			break;

	};
}

/**
 * Renders the version sub-pane of the current selected site pane
 */
function renderSiteVersionPane()
{
	var uuid = akeeba_remote_selected_sites[0];
	var siteSelector = '#site-' + uuid;
	var siteDef = siteManager.getSiteDefinition(uuid);
	
	var akeebaAPI = $(siteSelector).data('akeeba_api_object');
	var profileData = $(siteSelector).data('akeeba_profiles');
	var componentVersion = akeebaAPI.versionCache.component;
	var componentDate = akeebaAPI.versionCache.date;
	
	var profileDescription = '';
	profileDescription = profileData[akeebaAPI.profile];
	if(empty(profileDescription)) {
		if(!empty(profileData[1])) {
			profileDescription = ' &ndash; '+profileData[1];
		} else {
			profileDescription = '';
		}
	} else {
		profileDescription = ' &ndash; '+profileDescription;
	}
	
	// Update the info behind the scenes
	$('#siteInfoLRBDate').text(date(Trans.t('DATEFORMAT_LC1'),siteDef.lastSuccessfulBackup));
	$('#siteInfoLRBFile').text(siteDef.lastSuccessfulFilename);
	$('#siteInfoURL').text(akeebaAPI.URL);
	$('#siteInfoBackupProfile').html(akeebaAPI.profile + profileDescription);
	$('#siteInfoABVersion').html( componentVersion + ' <small>(' + componentDate + ')</small>' );
	// XXX Use Admin Tools' API to render Joomla! and AT version information...
	
	// Show the pane with a smooth effect
	$('div.idxPleaseWait').hide();
	$('#idxSiteInformation').show();
}

/**
 * Renders the backup in progress information pane
 */
function renderSiteBackupPane()
{
	var uuid = akeeba_remote_selected_sites[0];
	var siteSelector = '#site-' + uuid;
	
	var akeebaAPI = $(siteSelector).data('akeeba_api_object');
	var Domain = akeebaAPI.backupInfo.Domain;
	var Step = akeebaAPI.backupInfo.Step;
	var Substep = akeebaAPI.backupInfo.Substep;
	var Warnings = akeebaAPI.backupInfo.Warnings;
	
	var DomainString = '';
	
	switch(Domain)
	{
		case 'init':
			DomainString = Trans.t('STEPINIT');
			break;
		case 'installer':
			DomainString = Trans.t('STEPINSTALLER');
			break;
		case 'PackDB':
			DomainString = Trans.t('STEPPACKDB');
			break;
		case 'Packing':
			DomainString = Trans.t('STEPPACKING');
			break;
		case 'finale':
			DomainString = Trans.t('STEPFINALE');
			break;
		default:
			DomainString = sprintf(Trans.t('STEPDEFAULT'), Domain);
			break;
	}
	
	$('#backupDomain').text(DomainString);
	$('#backupStep').text(Step);
	$('#backupSubstep').text(Substep);
	
	// TODO Do something with the warnings
}

function renderSiteDownloadPane()
{
	var uuid = akeeba_remote_selected_sites[0];
	var siteSelector = '#site-' + uuid;
	
	var akeebaAPI = $(siteSelector).data('akeeba_api_object');

	var downloadedSize = formatBytes(akeebaAPI.DownloadDownloadedSize);
	var totalSize = formatBytes(akeebaAPI.DownloadTotalSize);
	var percentage = 0;
	
	if( (akeebaAPI.DownloadTotalSize > 0) && (akeebaAPI.DownloadDownloadedSize > 0) ) {
		percentage = 100 * akeebaAPI.DownloadDownloadedSize / akeebaAPI.DownloadTotalSize;
		var percentage = parseInt(percentage);
	}
	if(percentage < 0) percentage = 0;
	if(percentage > 100) percentage = 100;

	$('#downloadStatus').text( sprintf(Trans.t('DOWNLOADINGMSG'), downloadedSize, totalSize, percentage) );
	$('#downloadProgress').progressbar('value', percentage);
}

/**
 * Starts backing a site, defined by its UUID
 * @param uuid String The site to backup. Leave blank to start backing up the first selected site.
 * @return Boolean
 */
function startCurrentBackup(uuid)
{
	if(typeof(uuid) != 'string') {
		uuid = akeeba_remote_selected_sites[0];
	}
	var siteSelector = '#site-' + uuid;
	
	var akeebaAPI = $(siteSelector).data('akeeba_api_object');
	var status = siteManager.getSiteStatus(uuid);
	
	switch(status)
	{
		case 'backup':
		case 'download':
			// Sorry, can't start backing up a site in those states!
			return false;
			break;
			
		case 'queue':	
		default:
			// Are there more than x sites being backed up?
			var inBackup = siteManager.getNumSitesWithStatus('backup') * 1;
			var inDownload = siteManager.getNumSitesWithStatus('download') * 1;
			var inProgress = inBackup + inDownload;
			// FIXME Make the limit more flexible...
			if( inProgress >= 1 ) {
				akeebaAPI.enqueue();
				return;
			}
			
			// I can start backing up a site in idle or error state, though.
			// First, update the last backup time
			var siteDef = siteManager.getSiteDefinition(uuid);
			siteDef.lastBackup = time();
			siteManager.setSiteDefinition(uuid, siteDef);
			// Clear the site's log
			akeebaAPI.clearLog();
			// And start backing it up
			akeebaAPI.startBackup();
			break;
	}
	
	return true;
}

/**
 * Starts backing up all sites
 */
function startAllBackup()
{
	if(empty(akeeba_remote_selected_sites)) {
		// No sites selected? No problem, we'll add them all! Mwuahaha!
		akeeba_remote_selected_sites = [];
		$('#siteList li').each(function(idx, el){
			var siteID = $(el).attr('id');
			var uuid = substr(siteID, 5);
			array_push(akeeba_remote_selected_sites, uuid);
		});
	}
	$.each(akeeba_remote_selected_sites, function(index, uuid){
		startCurrentBackup(uuid);
	});
}

/**
 * Cancels backing up / downloading a site. If you skip the UUID it will cancel the first selected site.
 * @param uuid String
 * @return Boolean
 */
function cancelCurrentBackup(uuid)
{
	if(typeof(uuid) != 'string') {
		uuid = akeeba_remote_selected_sites[0];
	}
	var siteSelector = '#site-' + uuid;
	
	var akeebaAPI = $(siteSelector).data('akeeba_api_object');
	var status = siteManager.getSiteStatus(uuid);
	
	switch(status)
	{
		case 'queue':
		case 'backup':
			akeebaAPI.addLog('GUI: Sending cancelBackup signal');
			akeebaAPI.cancelBackup = true;
			break;
			
		case 'download':
			akeebaAPI.addLog('GUI: Sending cancelDownload signal');
			akeebaAPI.cancelDownload = true;
			break;
			
		default:
			// Cancel what? We're neither backing up nor downloading.
			return false;
			break;
	}
	
	return true;
}

/**
 * Cancel backing up / downloading on all selected sites
 * @return
 */
function cancelAllBackup()
{
	if(empty(akeeba_remote_selected_sites)) {
		// No sites selected? No problem, we'll add them all! Mwuahaha!
		akeeba_remote_selected_sites = [];
		$('#siteList li').each(function(idx, el){
			var siteID = $(el).attr('id');
			var uuid = substr(siteID, 5);
			array_push(akeeba_remote_selected_sites, uuid);
		});
	}
	$.each(akeeba_remote_selected_sites, function(index, uuid){
		cancelCurrentBackup(uuid);
	});	
}

/**
 * Launches the site editor to edit the currently selected site
 */
function editCurrentSite()
{
	uuid = akeeba_remote_selected_sites[0];
	if(!empty(uuid)) {
		siteEditorShow(uuid);
	}
}

/**
 * Asks the user and deletes the current site
 */
function deleteCurrentSite()
{
	uuid = akeeba_remote_selected_sites[0];
	var siteSelector = '#site-' + uuid;
	var siteName = $(siteSelector+' span.siteName:first').text();

	var answer = confirm( sprintf(Trans.t('DELETEPROMPT'), siteName));
	if(answer) {
		siteManager.removeSite(uuid);
		onSiteSelectionStop();
	}		
}

/**
 * Resets the errors on the current site
 */
function resetCurrentSiteErrors()
{
	uuid = akeeba_remote_selected_sites[0];
	siteManager.setSiteStatus(uuid, 'idle');
	onSiteSelectionStop();
}

/**
 * Shows the log for the current site
 */
function viewCurrentSiteLog()
{
	uuid = akeeba_remote_selected_sites[0];
	var siteSelector = '#site-' + uuid;
	var akeebaAPI = $(siteSelector).data('akeeba_api_object');
	
	siteLogViewerShow();
}

function cloneSite(uuid)
{
	// If no UUID is specified, use the current site
	if(empty(uuid)) {
		uuid = akeeba_remote_selected_sites[0];
	}
	if(empty(uuid)) return;
	
	// Get the old definition and save it under a new UUID
	var siteDef = siteManager.getSiteDefinition(uuid);
	var newUUID = createUUID();
	siteManager.addSite(newUUID, siteDef);
	
	// Save changes
	saveAppProperties();

	// Update GUI
	renderSingleSitePane();
}

/**
 * Periodic saving of application properties every 3 minutes
 */
setInterval("saveAppProperties()", 180000);