/**
 * Akeeba Remote Control
 * A utility to remotely control Akeeba Backup and Admin Tools
 * 
 * @version $Id$
 * @copyright 2008-2010 Nicholas K. Dionysopoulos / AkeebaBackup.com
 * @license GNU/GPL v3 or later
 * 
 * Scheduling (automation) control
 */

function schedulingTick()
{
	var sites = siteManager.siteDefinitions;
	if(empty(sites)) return;
	
	$.each(sites,function(uuid, data){
		if(data.scheduling)
		{
			if(empty(data.lastBackup)) data.lastBackup = 0;
			
			if(data.lastBackup == 0) {
				startCurrentBackup(uuid);
			} else {
				// Breaks it down to an object with keys: year, month, day, hour, minute, second
				var scheduleParts = date_parse(data.scheduleDateTime);
				var lastParts = date_parse(strftime('%Y-%m-%d %H:%M:%S', date.lastBackup));
				var scheduleDate = data.scheduleDateTime;
				var lastDate = data.lastBackup;
				var frequency = data.scheduleFrequency;
	
				if(frequency == 'daily') {
					var nextDay = strtotime('+1 day', lastDate);
					var nextParts = date_parse(strftime('%Y-%m-%d %H:%M:%S', nextDay));
					var checkDate = mktime(
						scheduleParts.hour, scheduleParts.minute, scheduleParts.second,
						nextParts.month, nextParts.day, nextParts.year 
					);
					
					Titanium.API.debug('Site '+uuid+' last backup '+strftime('%Y-%m-%d %H:%M',lastDate));
					
					if(time() > checkDate) startCurrentBackup(uuid);
				} else if(frequency == 'weekly') {
					// Try to find the next same date of week after last backup date.
					var nextDay = strtotime('+7 day', lastDate);
					
					// Zero out the hour/minute/second part of that date
					var nextParts = date_parse(strftime('%Y-%m-%d %H:%M:%S', nextDay));
					var checkDate = mktime(
						0, 0, 0,
						nextParts.month, nextParts.day, nextParts.year 
					);
					
					// Check if that date has elapsed. If so, start backing up.
					if(time() > checkDate) startCurrentBackup(uuid);
				} else if(frequency == 'monthly') {
					// Add a month to the last backup date
					var nextMonth = strtotime('+1 month', lastDate);
					
					// If the new month is greater than one and differs more than 1 from the last backup month, subtract 1 month
					var nextParts = date_parse(strftime('%Y-%m-%d %H:%M:%S', nextDay));
					var newYear = nextParts.year;
					var newMonth = nextParts.month;
					if( (newMonth > 1) && (newMonth - lastParts.month > 1) ) {
						newMonth = newMonth - 1;
					}
					
					// Change the day of month to be the spec'ed day of month.
					var newDay = scheduleParts.day;
					
					// If the spec'ed day of month is over 28 and the new month is 2, set it to 28
					if( (newMonth == 2) && (newDay > 28) ) newDay = 28;
					
					// If the spec'ed day of month is 31 and the month is 4,6,9,11 make it 30.
					if(newDay == 31) {
						if( (newMonth == 4) || (newMonth == 6) || (newMonth == 9) || (newMonth == 11) ) {
							newDay = 30;
						}
					}
					
					// Check date based on new month, day, year and at 00:00:00
					var checkDate = mktime(
							0, 0, 0,
							newMonth, newDay, newYear
						);				
					
					// Check if that date has elapsed. If so, start backing up.
					if(time() > checkDate) startCurrentBackup(uuid);
				} else if(frequency == 'yearly') {
					var newMonth = scheduleParts.month;
					var newYear = lastParts.year;
					var newDay = lastParts.day;
					
					var checkDate = mktime(
							0, 0, 0,
							newMonth, newDay, newYear
						);				
					
					// Check if that date has elapsed. If so, start backing up.
					if(time() > checkDate) startCurrentBackup(uuid);
				}
			}
		}
	});
}

// Fire the scheduling timer every 30 seconds
setInterval("schedulingTick()", 30000);