/**
 * Akeeba Remote Control
 * A utility to remotely control Akeeba Backup and Admin Tools
 * 
 * Akeeba Backup API class
 * @version $Id$
 */

/** @var String The UUID of the site we're editing. Set to null to create a new site */
var akeeba_remote_editor_uuid = null;
/** @var Object Caches the profiles list of the remote site */
var akeeba_remote_editor_profiles = null;
/** @var Boolean Flag to mark new sites; if false, we're editing an existing site */
var akeeba_remote_editor_newsite = false;
/** @var Object Holds the original site configuration */
var akeeba_remote_editor_sitedef = null;

function siteEditorShow(uuid)
{
	// Set the UUID of the site to edit
	if(empty(uuid)) uuid = null;
	akeeba_remote_editor_uuid = uuid;
	
	// Show the dialog and attach the button events
	$('#siteEditor').dialog({
		title: 'Site Editor',
		autoOpen: true,
		draggable: false,
		height: ($('body').height() - 40),
		width: ($('body').width() - 40),
		modal: true,
		open: function(event,ui){
			siteEditorHideError();			
			siteEditorInitialize();
			$('#siteEditorTabs').tabs('select',0);
		},
		close: function(event,ui) {
			siteEditorCancel();
		}
	});
}

/**
 * Initializes the site editor interface
 */
function siteEditorInitialize()
{
	// Initialize the internal variables
	akeeba_remote_editor_profiles = null;
	akeeba_remote_editor_newsite = false;
	akeeba_remote_editor_sitedef = null;
	
	// Reset the profiles drop-down
	$('#siteEditorProfile').html('');
	
	// Instantiate an API object for this site
	// --- If we have no UUID create a new one and add a blank site
	if(empty(akeeba_remote_editor_uuid)) {
		akeeba_remote_editor_newsite = true;
		akeeba_remote_editor_uuid = createUUID();
		var defaultSiteName = Trans.t('DEFAULTSITENAME');
		siteManager.addSite(akeeba_remote_editor_uuid, {
			name: defaultSiteName,
			URL:"http://",
			secret:"",
			profile: 1,
			download: false,
			downloadDirectory:"",
			deleteAfterDownload: 0,
			scheduling:false,
			scheduleFrequency:"daily",
			scheduleDateTime:"2010-01-01 00:00:00",
			lastBackup: 0,
			lastSuccessfulBackup: 0,
			lastSuccessfulFilename: ''
		});
	}
	// Put the site configuration options in the GUI
	var sitedef = siteManager.getSiteDefinition(akeeba_remote_editor_uuid);
	$('#siteEditorName').val(sitedef.name);
	$('#siteEditorURL').val(sitedef.URL);
	$('#siteEditorSecret').val(sitedef.secret);
	$('#siteEditorDownload').attr('checked', (sitedef.download ? 1 : 0) );
	$('#siteEditorDeleteAfterDownload').attr('checked', (sitedef.deleteAfterDownload ? 1 : 0) );
	$('#siteEditorDownloadDirectory').val(sitedef.downloadDirectory);
	$('#siteEditorScheduling').attr('checked', (sitedef.scheduling ? 1 : 0) );
	$('#siteEditorScheduleFrequency').val(sitedef.scheduleFrequency);
	$('#siteEditorScheduleDateTime').val(sitedef.scheduleDateTime);
	
	// Prepare GUI
	$("#siteEditorTabs").tabs();
	$('#siteEditorScheduleDateTime').datetimepicker();
	
	// Bind the event handlers
	$('#siteEditorName').keyup(siteEditorCheckFields);
	$('#siteEditorURL').keyup(siteEditorCheckFields);
	$('#siteEditorSecret').keyup(siteEditorCheckFields);
	//$('#siteEditorURL').blur(siteEditorAutoloadProfiles);
	//$('#siteEditorSecret').blur(siteEditorAutoloadProfiles);
	$('#siteEditorGetProfiles').click(siteEditorLoadProfiles);
	$('#siteEditorErrorClose').click(siteEditorHideError);
	
	// Run the initial field checking code
	siteEditorCheckFields();
	// If it is not a new site, load the profiles as well
	if(!akeeba_remote_editor_newsite) {
		siteEditorAutoloadProfiles();
	}
	akeeba_remote_editor_sitedef = sitedef;
}

/**
 * Checks the editor fields (Basics tab) for validity
 * @param {Object} e
 */
function siteEditorCheckFields(e)
{
	var hasSiteName = !empty($('#siteEditorName').val())
	var hasURL = siteEditorCheckURL();
	var hasSecret = !empty($('#siteEditorSecret').val());
	var buttonDisabled = !empty( $('#siteEditorGetProfiles').attr('disabled') );
	
	if(!hasSiteName) {
		$('#siteEditorName').css('border','thin solid red').css('background','#ffeeee');
	} else {
		$('#siteEditorName').css('border','thin solid gray').css('background','white');
	}		

	if(!hasURL) {
		$('#siteEditorURL').css('border','thin solid red').css('background','#ffeeee');
	} else {
		$('#siteEditorURL').css('border','thin solid gray').css('background','white');
	}		
	
	if(!hasSecret) {
		$('#siteEditorSecret').css('border','thin solid red').css('background','#ffeeee');
	} else {
		$('#siteEditorSecret').css('border','thin solid gray').css('background','white');
	}		

	if (hasURL && hasSecret) {
		if (buttonDisabled) {
			//$('#siteEditorGetProfiles').button('option', 'disabled', false);
			$('#siteEditorGetProfiles').attr('disabled','');
		}
	} else if(!buttonDisabled) {
		//$('#siteEditorGetProfiles').button('option','disabled', true);
		$('#siteEditorGetProfiles').attr('disabled','disabled');
	}
}

/**
 * Check the URL for validity and return the result as boolean
 * @return Boolean True if the URL is valid
 */
function siteEditorCheckURL()
{
	var url = $('#siteEditorURL').val();
	var isValid = true;
	
	if(!empty(url))
	{
		// Check the protocol. It must be http:// or https://
		var protocol = strtolower(substr(url,0,7));
		if(protocol != 'http://') {
			protocol = strtolower(substr(url,0,8));
			if(protocol != 'https://') {
				isValid = false;
			}
		}
		
		// Get the domain name and validate it
		var domain = substr(url, strlen(protocol));
		var firstSlash = strpos(domain,'/');
		if(firstSlash !== false) {
			domain = substr(domain, 0, firstSlash);
		}
		isValid = isValid && ( /^([a-z0-9_\-]+\.)*[a-z]{2,4}$/i.test(domain) || (domain == 'localhost') );
	} else {
		isValid = false;
	}
	
	return isValid;
}

/**
 * Autoloads the profiles list
 */
function siteEditorAutoloadProfiles()
{
	if( akeeba_remote_editor_profiles !== null ) return;
	var hasURL = siteEditorCheckURL();
	var hasSecret = !empty($('#siteEditorSecret').val());

	if(hasURL && hasSecret) {
		siteEditorLoadProfiles();
	}			
}

/**
 * Loads the profiles list
 */
function siteEditorLoadProfiles()
{
	// Disable the edit boxes
	$('#siteEditorName').attr('disabled','disabled');
	$('#siteEditorURL').attr('disabled','disabled');
	$('#siteEditorSecret').attr('disabled','disabled');
	$('#siteEditorProfile').attr('disabled','disabled');
	$('#siteEditorGetProfiles').attr('disabled','disabled');
	
	// Unfocus the edit boxes
	$('#siteEditorGetProfiles').focus();
	
	// Send a request to the site to load the profiles
	// -- Fetch the site API object
	var siteSelector = '#site-' + akeeba_remote_editor_uuid;
	var akeebaAPI = $(siteSelector).data('akeeba_api_object');
	// -- Feed it with the new data
	akeebaAPI.URL = $('#siteEditorURL').val();
	akeebaAPI.secret = $('#siteEditorSecret').val();

	akeebaAPI.runMethod('getProfiles', {}, siteEditorGetProfiles_cb, 7000);			
}

/**
 * Callback for the getProfiles AJAX request
 * @param data Parsed JSON data
 * @param uuid The UUID of the site being loaded
 */
function siteEditorGetProfiles_cb(data, uuid)
{
	var siteSelector = '#site-' + uuid;
	var akeebaAPI = $(siteSelector).data('akeeba_api_object');
	var errorMessage = '';
	
	if(data.status != 200) {
		// An error occurred; get the error message
		if(empty(data.msg)) {
			errorMessage = akeebaAPI.errormessages[data.status];
		} else {
			errorMessage = data.msg;
		}
		$('#siteEditorProfile').attr('disabled','');
		akeeba_remote_editor_profiles = null;
		siteEditorShowError(errorMessage);
	} else {
		// Cache the data
		akeeba_remote_editor_profiles = data.msg;
		// Enable the drop-down
		$('#siteEditorProfile').attr('disabled','');
		// Populate the drop-down
		var sitedef = siteManager.getSiteDefinition(uuid);
		$('#siteEditorProfile').html('');
		$.each(akeeba_remote_editor_profiles,function(index,item){
			var element = $(document.createElement('option'))
				.attr('value',item.id)
				.text(item.id + ' - '+item.name);
			if(item.id == sitedef.profile) {
				element.attr('selected','selected');
			}
			element.appendTo($('#siteEditorProfile'));
		});
	}

	// Re-enable the edit boxes
	$('#siteEditorName').attr('disabled','');
	$('#siteEditorURL').attr('disabled','');
	$('#siteEditorSecret').attr('disabled','');
	$('#siteEditorGetProfiles').attr('disabled','');
	// Hide the spinner
	// $('#siteEditorLoading').hide();
}

/**
 * Shows the error slidebox in the site editor GUI
 * @param {String} errorMessage The error message to display
 */
function siteEditorShowError(errorMessage)
{			
	$('#siteEditorErrorMessage').html(errorMessage);
	$('#siteEditorError').css('top', $('#siteEditor').position().top );
	$('#siteEditorError').height( $('#siteEditor').height() - 55 );
	$('#siteEditorError').show();
}

/**
 * Hides the error slidebox in the site editor GUI
 */
function siteEditorHideError()
{
	$('#siteEditorError').hide();
}

/**
 * Shows the directory browser
 * @return
 */
function siteEditorDLBrowse()
{
	var initialDirectory = $('#siteEditorDownloadDirectory').val();
	if(empty(initialDirectory)) initialDirectory = Titanium.Filesystem.getDocumentsDirectory();
	
	var windowTitle = Trans.t('CHOOSEOUTPUTDIR');
	var params = {
		title: windowTitle,
		path: initialDirectory,
		multiple: false
	};
	Titanium.UI.currentWindow.openFolderChooserDialog(function(f){
		if(f.length) {
			$('#siteEditorDownloadDirectory').val(f[0]);
		}
	},params);
}

/**
 * Applies the changes made in the editor
 */
function siteEditorApply()
{
	var sitedef = akeeba_remote_editor_sitedef;
	// Basic options
	sitedef.name = $('#siteEditorName').val();
	sitedef.URL = $('#siteEditorURL').val();
	sitedef.secret = $('#siteEditorSecret').val();
	sitedef.profile = $('#siteEditorProfile').val();
	// Download
	sitedef.download = $('#siteEditorDownload').is(':checked');
	sitedef.downloadDirectory = $('#siteEditorDownloadDirectory').val();
	sitedef.deleteAfterDownload = $('#siteEditorDeleteAfterDownload').is(':checked') ? 1 : 0;
	// Scheduling
	sitedef.scheduling = $('#siteEditorScheduling').is(':checked'); 
	sitedef.scheduleFrequency = $('#siteEditorScheduleFrequency').val();
	sitedef.scheduleDateTime = $('#siteEditorScheduleDateTime').val();

	siteManager.setSiteDefinition(akeeba_remote_editor_uuid, sitedef);
	
	var siteSelector = '#site-' + akeeba_remote_editor_uuid;
	var akeebaAPI = $(siteSelector).data('akeeba_api_object');
	akeebaAPI.loadSite(akeeba_remote_editor_uuid);
	
	akeeba_remote_editor_uuid = null;
	$('#siteEditor').dialog('close');
	
	if(akeebaAPI.isCurrent()) renderSingleSitePane();
	
	saveAppProperties();
}

/**
 * Reverts any changes made in the editor
 */
function siteEditorCancel()
{
	if(akeeba_remote_editor_uuid == null) return;
	
	if(akeeba_remote_editor_newsite) {
		// remove the new site definition
		siteManager.removeSite(akeeba_remote_editor_uuid);
	} else {
		// revert changes on existing sites
		siteManager.setSiteDefinition(akeeba_remote_editor_uuid, akeeba_remote_editor_sitedef);
		
		var siteSelector = '#site-' + akeeba_remote_editor_uuid;
		var akeebaAPI = $(siteSelector).data('akeeba_api_object');
		akeebaAPI.loadSite(akeeba_remote_editor_uuid);							
	}
	
	akeeba_remote_editor_uuid = null;
	$('#siteEditor').dialog('close');
	
	if(akeebaAPI.isCurrent()) renderSingleSitePane();
}
