/**
 * Akeeba Remote Control
 * A utility to remotely control Akeeba Backup and Admin Tools
 * 
 * @version $Id$
 * @copyright 2008-2010 Nicholas K. Dionysopoulos / AkeebaBackup.com
 * @license GNU/GPL v3 or later
 */

/**
 * Translation API prototype
 */
function ARCText()
{
	this.preferredLanguage = 'en-GB';
	this.translations = {};
};

/**
 * Dump the translation strings to a JSON file
 * @return
 */
ARCText.prototype.dump = function()
{
	// Fetch translation keys
	var translations = {};
	$.each(	$('[translatekey]'), function(index, el) {
		var key = $(el).attr('translatekey')
		var value = $(el).text();
		translations[key] = value;
	} );
	var jsonized = json_encode(translations);
	// Save to file
	var outdir = Titanium.Filesystem.getDesktopDirectory();
	var file = Titanium.Filesystem.getFile(outdir,'default.json');
	var stream = Titanium.Filesystem.getFileStream(file);
	stream.open(Titanium.Filesystem.MODE_WRITE, false, false);
	stream.write(jsonized);
	stream.close();
};

ARCText.prototype.load = function()
{
	var translations = {};
	var transPath = Titanium.Filesystem.getFile( Titanium.Filesystem.getResourcesDirectory(), 'languages' );
	var filelist = transPath.getDirectoryListing();
	
	$.each(filelist, function(index, filename){
		var stream = Titanium.Filesystem.getFileStream(filename);
		stream.open(Titanium.Filesystem.MODE_READ, false, false);
		var jsonized = '';
		var line = stream.readLine();
		while(line != null) {
			jsonized += line+"\n";
			line = stream.readLine();
		}
		if(!empty(jsonized)) {
			var filepath = Titanium.Filesystem.getFile(filename).toString();
			var key = basename(filepath,'.json');
			var transobj = json_decode(jsonized);
			if(!empty(transobj)) {
				translations[key] = transobj;
			}
		}
	});
	
	this.translations = translations;
};

ARCText.prototype.makeDropdown = function()
{
	$('#idxSelectLanguage').html('');
	$(document.createElement('option'))
		.attr('value', 'default')
		.text('-----')
		.appendTo('#idxSelectLanguage');
	$.each(this.translations, function(key,trans){
		$(document.createElement('option'))
			.attr('value', key)
			.text(trans.ARC_LANGUAGE_NAME)
			.appendTo('#idxSelectLanguage');
	});
};

ARCText.prototype.apply = function(newLanguage)
{
	if(newLanguage == 'default') newLanguage = 'en-GB';
	Trans.preferredLanguage = newLanguage;
	$.each(	$('[translatekey]'), function(index, el) {
		var key = $(el).attr('translatekey')
		var value = Trans.t(key);
		$(el).text(value);
	} );
};

ARCText.prototype.t = function(key)
{
	var lang = this.preferredLanguage;
	var trans = this.translations[lang];
	if(empty(trans)) {
		return key;
	}
	var value = trans[key];
	if(empty(value)) {
		return key;
	}
	return value;
};

var Trans = new ARCText();

$(document).ready(function(){
	Trans.load();
	Trans.makeDropdown();
	
	$('#idxSelectLanguage').change(function(){
		var lang = $('#idxSelectLanguage').val();
		Trans.apply(lang);
	});
});